#pragma once

#include <stdint.h>

namespace uart {

enum class DataBits {
	FIVE,
	SIX,
	SEVEN,
	EIGHT,
	NINE,
};

enum class StopBits {
	ONE,
	TWO,
};

enum class Parity {
	NONE,
	ODD,
	EVEN,
};

namespace detail {

template <DataBits dataBits>
struct choose_data_type {
	using type = uint8_t;
};

template <>
struct choose_data_type<DataBits::NINE> {
	using type = uint16_t;
};

} // namespace detail

template <uint32_t baudRate = 9600, DataBits dataBits = DataBits::EIGHT, Parity parity = Parity::NONE,
          StopBits stopBits = StopBits::ONE>
struct Config {
	static constexpr auto BAUD_RATE = baudRate;
	static constexpr auto DATA_BITS = dataBits;
	static constexpr auto PARITY = parity;
	static constexpr auto STOP_BITS = stopBits;
	using data_t = typename detail::choose_data_type<DATA_BITS>::type;
};

} // namespace uart
